# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.payment import (
    AutoRechargePrefListResponse,
    AutoRechargePrefUpdateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAutoRechargePrefs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        auto_recharge_pref = client.payment.auto_recharge_prefs.update()
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        auto_recharge_pref = client.payment.auto_recharge_prefs.update(
            enabled=True,
            invoice_enabled=True,
            preference="credit_paypal",
            recharge_amount="104.00",
            threshold_amount="104.00",
        )
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.payment.auto_recharge_prefs.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auto_recharge_pref = response.parse()
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.payment.auto_recharge_prefs.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auto_recharge_pref = response.parse()
            assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        auto_recharge_pref = client.payment.auto_recharge_prefs.list()
        assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.payment.auto_recharge_prefs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auto_recharge_pref = response.parse()
        assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.payment.auto_recharge_prefs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auto_recharge_pref = response.parse()
            assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAutoRechargePrefs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        auto_recharge_pref = await async_client.payment.auto_recharge_prefs.update()
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        auto_recharge_pref = await async_client.payment.auto_recharge_prefs.update(
            enabled=True,
            invoice_enabled=True,
            preference="credit_paypal",
            recharge_amount="104.00",
            threshold_amount="104.00",
        )
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.payment.auto_recharge_prefs.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auto_recharge_pref = await response.parse()
        assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.payment.auto_recharge_prefs.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auto_recharge_pref = await response.parse()
            assert_matches_type(AutoRechargePrefUpdateResponse, auto_recharge_pref, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        auto_recharge_pref = await async_client.payment.auto_recharge_prefs.list()
        assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.payment.auto_recharge_prefs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auto_recharge_pref = await response.parse()
        assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.payment.auto_recharge_prefs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auto_recharge_pref = await response.parse()
            assert_matches_type(AutoRechargePrefListResponse, auto_recharge_pref, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["UserAddress"]


class UserAddress(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the user address."""

    administrative_area: Optional[str] = None
    """The locality of the user address.

    For US addresses, this corresponds to the state of the address.
    """

    borough: Optional[str] = None
    """The borough of the user address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    business_name: Optional[str] = None
    """The business name associated with the user address."""

    country_code: Optional[str] = None
    """The two-character (ISO 3166-1 alpha-2) country code of the user address."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    extended_address: Optional[str] = None
    """
    Additional street address information about the user address such as, but not
    limited to, unit number or apartment number.
    """

    first_name: Optional[str] = None
    """The first name associated with the user address."""

    last_name: Optional[str] = None
    """The last name associated with the user address."""

    locality: Optional[str] = None
    """The locality of the user address.

    For US addresses, this corresponds to the city of the address.
    """

    neighborhood: Optional[str] = None
    """The neighborhood of the user address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    phone_number: Optional[str] = None
    """The phone number associated with the user address."""

    postal_code: Optional[str] = None
    """The postal code of the user address."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    street_address: Optional[str] = None
    """The primary street address information about the user address."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job import Job as Job
from .job_error import JobError as JobError
from .job_list_params import JobListParams as JobListParams
from .job_retrieve_response import JobRetrieveResponse as JobRetrieveResponse
from .job_delete_phone_number_block_params import JobDeletePhoneNumberBlockParams as JobDeletePhoneNumberBlockParams
from .job_delete_phone_number_block_response import (
    JobDeletePhoneNumberBlockResponse as JobDeletePhoneNumberBlockResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from ...._models import BaseModel

__all__ = ["ToolTestResponse", "Data"]


class Data(BaseModel):
    """Response model for webhook tool test results"""

    content_type: str

    request: Dict[str, object]

    response: str

    status_code: int

    success: bool


class ToolTestResponse(BaseModel):
    """Response model for webhook tool test results"""

    data: Data
    """Response model for webhook tool test results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["NotificationSetting", "Parameter"]


class Parameter(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None


class NotificationSetting(BaseModel):
    id: Optional[str] = None
    """A UUID."""

    associated_record_type: Optional[str] = None

    associated_record_type_value: Optional[str] = None

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    notification_channel_id: Optional[str] = None
    """A UUID reference to the associated Notification Channel."""

    notification_event_condition_id: Optional[str] = None
    """A UUID reference to the associated Notification Event Condition."""

    notification_profile_id: Optional[str] = None
    """A UUID reference to the associated Notification Profile."""

    parameters: Optional[List[Parameter]] = None

    status: Optional[
        Literal[
            "enabled",
            "enable-received",
            "enable-pending",
            "enable-submtited",
            "delete-received",
            "delete-pending",
            "delete-submitted",
            "deleted",
        ]
    ] = None
    """Most preferences apply immediately; however, other may needs to propagate."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.ai.assistants.tests import TestRunResponse
from telnyx.types.ai.assistants.tests.test_suites import RunTriggerResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRuns:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        run = client.ai.assistants.tests.test_suites.runs.list(
            suite_name="suite_name",
        )
        assert_matches_type(SyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        run = client.ai.assistants.tests.test_suites.runs.list(
            suite_name="suite_name",
            page_number=0,
            page_size=0,
            status="status",
            test_suite_run_id="test_suite_run_id",
        )
        assert_matches_type(SyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.assistants.tests.test_suites.runs.with_raw_response.list(
            suite_name="suite_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.assistants.tests.test_suites.runs.with_streaming_response.list(
            suite_name="suite_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `suite_name` but received ''"):
            client.ai.assistants.tests.test_suites.runs.with_raw_response.list(
                suite_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger(self, client: Telnyx) -> None:
        run = client.ai.assistants.tests.test_suites.runs.trigger(
            suite_name="suite_name",
        )
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_with_all_params(self, client: Telnyx) -> None:
        run = client.ai.assistants.tests.test_suites.runs.trigger(
            suite_name="suite_name",
            destination_version_id="123e4567-e89b-12d3-a456-426614174000",
        )
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_trigger(self, client: Telnyx) -> None:
        response = client.ai.assistants.tests.test_suites.runs.with_raw_response.trigger(
            suite_name="suite_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_trigger(self, client: Telnyx) -> None:
        with client.ai.assistants.tests.test_suites.runs.with_streaming_response.trigger(
            suite_name="suite_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(RunTriggerResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_trigger(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `suite_name` but received ''"):
            client.ai.assistants.tests.test_suites.runs.with_raw_response.trigger(
                suite_name="",
            )


class TestAsyncRuns:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        run = await async_client.ai.assistants.tests.test_suites.runs.list(
            suite_name="suite_name",
        )
        assert_matches_type(AsyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        run = await async_client.ai.assistants.tests.test_suites.runs.list(
            suite_name="suite_name",
            page_number=0,
            page_size=0,
            status="status",
            test_suite_run_id="test_suite_run_id",
        )
        assert_matches_type(AsyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.tests.test_suites.runs.with_raw_response.list(
            suite_name="suite_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.tests.test_suites.runs.with_streaming_response.list(
            suite_name="suite_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[TestRunResponse], run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `suite_name` but received ''"):
            await async_client.ai.assistants.tests.test_suites.runs.with_raw_response.list(
                suite_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger(self, async_client: AsyncTelnyx) -> None:
        run = await async_client.ai.assistants.tests.test_suites.runs.trigger(
            suite_name="suite_name",
        )
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_with_all_params(self, async_client: AsyncTelnyx) -> None:
        run = await async_client.ai.assistants.tests.test_suites.runs.trigger(
            suite_name="suite_name",
            destination_version_id="123e4567-e89b-12d3-a456-426614174000",
        )
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_trigger(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.tests.test_suites.runs.with_raw_response.trigger(
            suite_name="suite_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(RunTriggerResponse, run, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_trigger(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.tests.test_suites.runs.with_streaming_response.trigger(
            suite_name="suite_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(RunTriggerResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_trigger(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `suite_name` but received ''"):
            await async_client.ai.assistants.tests.test_suites.runs.with_raw_response.trigger(
                suite_name="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_siprec_started import CallSiprecStarted

__all__ = ["CallSiprecStartedWebhookEvent"]


class CallSiprecStartedWebhookEvent(BaseModel):
    data: Optional[CallSiprecStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EmbeddingURLParams"]


class EmbeddingURLParams(TypedDict, total=False):
    bucket_name: Required[str]
    """Name of the bucket to store the embeddings. This bucket must already exist."""

    url: Required[str]
    """The URL of the webpage to embed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "WebhookDeliveryListParams",
    "Filter",
    "FilterAttempts",
    "FilterFinishedAt",
    "FilterStartedAt",
    "FilterStatus",
    "FilterWebhook",
]


class WebhookDeliveryListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status][eq], filter[event_type], filter[webhook][contains],
    filter[attempts][contains], filter[started_at][gte], filter[started_at][lte],
    filter[finished_at][gte], filter[finished_at][lte]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterAttempts(TypedDict, total=False):
    contains: str
    """
    Return only webhook_deliveries whose `attempts` component contains the given
    text
    """


class FilterFinishedAt(TypedDict, total=False):
    gte: str
    """
    Return only webhook_deliveries whose delivery finished later than or at given
    ISO 8601 datetime
    """

    lte: str
    """
    Return only webhook_deliveries whose delivery finished earlier than or at given
    ISO 8601 datetime
    """


class FilterStartedAt(TypedDict, total=False):
    gte: str
    """
    Return only webhook_deliveries whose delivery started later than or at given ISO
    8601 datetime
    """

    lte: str
    """
    Return only webhook_deliveries whose delivery started earlier than or at given
    ISO 8601 datetime
    """


class FilterStatus(TypedDict, total=False):
    eq: Literal["delivered", "failed"]
    """Return only webhook_deliveries matching the given `status`"""


class FilterWebhook(TypedDict, total=False):
    contains: str
    """
    Return only webhook deliveries whose `webhook` component contains the given text
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status][eq], filter[event_type], filter[webhook][contains], filter[attempts][contains], filter[started_at][gte], filter[started_at][lte], filter[finished_at][gte], filter[finished_at][lte]
    """

    attempts: FilterAttempts

    event_type: str
    """Return only webhook_deliveries matching the given value of `event_type`.

    Accepts multiple values separated by a `,`.
    """

    finished_at: FilterFinishedAt

    started_at: FilterStartedAt

    status: FilterStatus

    webhook: FilterWebhook



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TelephonyCredentialUpdateParams"]


class TelephonyCredentialUpdateParams(TypedDict, total=False):
    connection_id: str
    """Identifies the Credential Connection this credential is associated with."""

    expires_at: str
    """ISO-8601 formatted date indicating when the credential will expire."""

    name: str

    tag: str
    """Tags a credential. A single tag can hold at maximum 1000 credentials."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["TextToSpeechGenerateParams", "Aws", "Azure", "Elevenlabs", "Minimax", "Resemble", "Rime", "Telnyx"]


class TextToSpeechGenerateParams(TypedDict, total=False):
    aws: Aws
    """AWS Polly provider-specific parameters."""

    azure: Azure
    """Azure Cognitive Services provider-specific parameters."""

    disable_cache: bool
    """When `true`, bypass the audio cache and generate fresh audio."""

    elevenlabs: Elevenlabs
    """ElevenLabs provider-specific parameters."""

    inworld: Dict[str, object]
    """Inworld provider-specific parameters."""

    language: str
    """Language code (e.g. `en-US`). Usage varies by provider."""

    minimax: Minimax
    """Minimax provider-specific parameters."""

    output_type: Literal["binary_output", "base64_output"]
    """Determines the response format.

    `binary_output` returns raw audio bytes, `base64_output` returns base64-encoded
    audio in JSON.
    """

    provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
    """TTS provider. Required unless `voice` is provided."""

    resemble: Resemble
    """Resemble AI provider-specific parameters."""

    rime: Rime
    """Rime provider-specific parameters."""

    telnyx: Telnyx
    """Telnyx provider-specific parameters."""

    text: str
    """The text to convert to speech."""

    text_type: Literal["text", "ssml"]
    """Text type. Use `ssml` for SSML-formatted input (supported by AWS and Azure)."""

    voice: str
    """
    Voice identifier in the format `provider.model_id.voice_id` or
    `provider.voice_id`. Examples: `telnyx.NaturalHD.Alloy`,
    `azure.en-US-AvaMultilingualNeural`, `aws.Polly.Generative.Lucia`. When
    provided, `provider`, `model_id`, and `voice_id` are extracted automatically and
    take precedence over individual parameters.
    """

    voice_settings: Dict[str, object]
    """Provider-specific voice settings.

    Contents vary by provider — see provider-specific parameter objects below.
    """


class Aws(TypedDict, total=False):
    """AWS Polly provider-specific parameters."""

    language_code: str
    """Language code (e.g. `en-US`, `es-ES`)."""

    lexicon_names: SequenceNotStr[str]
    """List of lexicon names to apply."""

    output_format: str
    """Audio output format."""

    sample_rate: str
    """Audio sample rate."""

    text_type: Literal["text", "ssml"]
    """Input text type."""


class Azure(TypedDict, total=False):
    """Azure Cognitive Services provider-specific parameters."""

    api_key: str
    """Custom Azure API key. If not provided, the default Telnyx key is used."""

    deployment_id: str
    """Custom Azure deployment ID."""

    effect: str
    """Azure audio effect to apply."""

    gender: str
    """Voice gender preference."""

    language_code: str
    """Language code (e.g. `en-US`)."""

    output_format: str
    """Azure audio output format."""

    region: str
    """Azure region (e.g. `eastus`, `westeurope`)."""

    text_type: Literal["text", "ssml"]
    """Input text type. Use `ssml` for SSML-formatted input."""


class Elevenlabs(TypedDict, total=False):
    """ElevenLabs provider-specific parameters."""

    api_key: str
    """Custom ElevenLabs API key. If not provided, the default Telnyx key is used."""

    language_code: str
    """Language code."""

    voice_settings: Dict[str, object]
    """ElevenLabs voice settings (stability, similarity_boost, etc.)."""


class Minimax(TypedDict, total=False):
    """Minimax provider-specific parameters."""

    language_boost: str
    """Language code to boost pronunciation for."""

    pitch: int
    """Pitch adjustment."""

    response_format: str
    """Audio output format."""

    speed: float
    """Speech speed multiplier."""

    vol: float
    """Volume level."""


class Resemble(TypedDict, total=False):
    """Resemble AI provider-specific parameters."""

    api_key: str
    """Custom Resemble API key."""

    format: str
    """Audio output format."""

    precision: str
    """Synthesis precision."""

    sample_rate: str
    """Audio sample rate."""


class Rime(TypedDict, total=False):
    """Rime provider-specific parameters."""

    response_format: str
    """Audio output format."""

    sampling_rate: int
    """Audio sampling rate in Hz."""

    voice_speed: float
    """Voice speed multiplier."""


class Telnyx(TypedDict, total=False):
    """Telnyx provider-specific parameters."""

    response_format: str
    """Audio response format."""

    sampling_rate: int
    """Audio sampling rate in Hz."""

    temperature: float
    """Sampling temperature."""

    voice_speed: float
    """Voice speed multiplier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionStopPlaybackParams"]


class ActionStopPlaybackParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    overlay: bool
    """When enabled, it stops the audio being played in the overlay queue."""

    stop: str
    """Use `current` to stop the current audio being played.

    Use `all` to stop the current audio file being played and clear all audio files
    from the queue.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Location"]


class Location(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the resource."""

    additional_info: Optional[str] = None

    description: Optional[str] = None

    is_default: Optional[bool] = None
    """Represents whether the location is the default or not."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from typing_extensions import TypeAlias

from .campaign_sharing_status import CampaignSharingStatus

__all__ = ["PartnerCampaignRetrieveSharingStatusResponse"]

PartnerCampaignRetrieveSharingStatusResponse: TypeAlias = Dict[str, CampaignSharingStatus]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingEventDeletedPayload", "Payload"]


class Payload(BaseModel):
    id: Optional[str] = None
    """Identifies the porting order that was deleted."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    deleted_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the porting order was deleted."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class PortingEventDeletedPayload(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    event_type: Optional[Literal["porting_order.deleted"]] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CallRecordingParam"]


class CallRecordingParam(TypedDict, total=False):
    """The call recording settings for a phone number."""

    inbound_call_recording_channels: Literal["single", "dual"]
    """
    When using 'dual' channels, final audio file will be stereo recorded with the
    first leg on channel A, and the rest on channel B.
    """

    inbound_call_recording_enabled: bool
    """When enabled, any inbound call to this number will be recorded."""

    inbound_call_recording_format: Literal["wav", "mp3"]
    """The audio file format for calls being recorded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TfVerificationStatus"]

TfVerificationStatus: TypeAlias = Literal[
    "Verified", "Rejected", "Waiting For Vendor", "Waiting For Customer", "Waiting For Telnyx", "In Progress"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PortingPhoneNumberListParams", "Filter"]


class PortingPhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_order_status]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_order_status]
    """

    porting_order_status: Literal[
        "draft", "in-process", "submitted", "exception", "foc-date-confirmed", "cancel-pending", "ported", "cancelled"
    ]
    """Filter results by porting order status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, TypedDict

from .whatsapp_media_param import WhatsappMediaParam
from .whatsapp_contact_param import WhatsappContactParam
from .whatsapp_location_param import WhatsappLocationParam
from .whatsapp_reaction_param import WhatsappReactionParam
from .whatsapp_interactive_param import WhatsappInteractiveParam

__all__ = ["WhatsappMessageContentParam"]


class WhatsappMessageContentParam(TypedDict, total=False):
    audio: WhatsappMediaParam

    biz_opaque_callback_data: str
    """custom data to return with status update"""

    contacts: Iterable[WhatsappContactParam]

    document: WhatsappMediaParam

    image: WhatsappMediaParam

    interactive: WhatsappInteractiveParam

    location: WhatsappLocationParam

    reaction: WhatsappReactionParam

    sticker: WhatsappMediaParam

    type: Literal[
        "audio", "document", "image", "sticker", "video", "interactive", "location", "template", "reaction", "contacts"
    ]

    video: WhatsappMediaParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ExternalVettingOrderParams"]


class ExternalVettingOrderParams(TypedDict, total=False):
    evp_id: Required[Annotated[str, PropertyInfo(alias="evpId")]]
    """External vetting provider ID for the brand."""

    vetting_class: Required[Annotated[str, PropertyInfo(alias="vettingClass")]]
    """Identifies the vetting classification."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["MessagingSettings"]


class MessagingSettings(BaseModel):
    conversation_inactivity_minutes: Optional[int] = None
    """
    If more than this many minutes have passed since the last message, the assistant
    will start a new conversation instead of continuing the existing one.
    """

    default_messaging_profile_id: Optional[str] = None
    """Default Messaging Profile used for messaging exchanges with your assistant.

    This will be created automatically on assistant creation.
    """

    delivery_status_webhook_url: Optional[str] = None
    """
    The URL where webhooks related to delivery statused for assistant messages will
    be sent.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SimCardActionsSummary"]


class SimCardActionsSummary(BaseModel):
    count: Optional[int] = None

    status: Optional[Literal["in-progress", "completed", "failed", "interrupted"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["InboundChannelUpdateResponse", "Data"]


class Data(BaseModel):
    channels: Optional[int] = None
    """The number of channels set for the account"""

    record_type: Optional[str] = None
    """Identifies the type of the response"""


class InboundChannelUpdateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from .plan_step_data import PlanStepData

__all__ = ["PlanUpdateStepResponse"]


class PlanUpdateStepResponse(BaseModel):
    data: PlanStepData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.actions import register_create_params
from ...types.actions.register_create_response import RegisterCreateResponse

__all__ = ["RegisterResource", "AsyncRegisterResource"]


class RegisterResource(SyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> RegisterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RegisterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegisterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RegisterResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        registration_codes: SequenceNotStr[str],
        sim_card_group_id: str | Omit = omit,
        status: Literal["enabled", "disabled", "standby"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegisterCreateResponse:
        """
        Register the SIM cards associated with the provided registration codes to the
        current user's account.<br/><br/> If <code>sim_card_group_id</code> is provided,
        the SIM cards will be associated with that group. Otherwise, the default group
        for the current user will be used.<br/><br/>

        Args:
          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          status: Status on which the SIM card will be set after being successful registered.

          tags: Searchable tags associated with the SIM card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/actions/register/sim_cards",
            body=maybe_transform(
                {
                    "registration_codes": registration_codes,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                },
                register_create_params.RegisterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegisterCreateResponse,
        )


class AsyncRegisterResource(AsyncAPIResource):
    """SIM Cards operations"""

    @cached_property
    def with_raw_response(self) -> AsyncRegisterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegisterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegisterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRegisterResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        registration_codes: SequenceNotStr[str],
        sim_card_group_id: str | Omit = omit,
        status: Literal["enabled", "disabled", "standby"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegisterCreateResponse:
        """
        Register the SIM cards associated with the provided registration codes to the
        current user's account.<br/><br/> If <code>sim_card_group_id</code> is provided,
        the SIM cards will be associated with that group. Otherwise, the default group
        for the current user will be used.<br/><br/>

        Args:
          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          status: Status on which the SIM card will be set after being successful registered.

          tags: Searchable tags associated with the SIM card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/actions/register/sim_cards",
            body=await async_maybe_transform(
                {
                    "registration_codes": registration_codes,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                },
                register_create_params.RegisterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegisterCreateResponse,
        )


class RegisterResourceWithRawResponse:
    def __init__(self, register: RegisterResource) -> None:
        self._register = register

        self.create = to_raw_response_wrapper(
            register.create,
        )


class AsyncRegisterResourceWithRawResponse:
    def __init__(self, register: AsyncRegisterResource) -> None:
        self._register = register

        self.create = async_to_raw_response_wrapper(
            register.create,
        )


class RegisterResourceWithStreamingResponse:
    def __init__(self, register: RegisterResource) -> None:
        self._register = register

        self.create = to_streamed_response_wrapper(
            register.create,
        )


class AsyncRegisterResourceWithStreamingResponse:
    def __init__(self, register: AsyncRegisterResource) -> None:
        self._register = register

        self.create = async_to_streamed_response_wrapper(
            register.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WdrReport"]


class WdrReport(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    end_time: Optional[str] = None
    """ISO 8601 formatted date-time indicating the end time."""

    record_type: Optional[str] = None

    report_url: Optional[str] = None
    """The URL where the report content, when generated, will be published to."""

    start_time: Optional[str] = None
    """ISO 8601 formatted date-time indicating the start time."""

    status: Optional[Literal["pending", "complete", "failed", "deleted"]] = None
    """Indicates the status of the report, which is updated asynchronously."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EmbeddingSimilaritySearchParams"]


class EmbeddingSimilaritySearchParams(TypedDict, total=False):
    bucket_name: Required[str]

    query: Required[str]

    num_of_docs: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .audio_visualizer_config import AudioVisualizerConfig

__all__ = ["WidgetSettings"]


class WidgetSettings(BaseModel):
    """Configuration settings for the assistant's web widget."""

    agent_thinking_text: Optional[str] = None
    """Text displayed while the agent is processing."""

    audio_visualizer_config: Optional[AudioVisualizerConfig] = None

    default_state: Optional[Literal["expanded", "collapsed"]] = None
    """The default state of the widget."""

    give_feedback_url: Optional[str] = None
    """URL for users to give feedback."""

    logo_icon_url: Optional[str] = None
    """URL to a custom logo icon for the widget."""

    position: Optional[Literal["fixed", "static"]] = None
    """The positioning style for the widget."""

    report_issue_url: Optional[str] = None
    """URL for users to report issues."""

    speak_to_interrupt_text: Optional[str] = None
    """Text prompting users to speak to interrupt."""

    start_call_text: Optional[str] = None
    """Custom text displayed on the start call button."""

    theme: Optional[Literal["light", "dark"]] = None
    """The visual theme for the widget."""

    view_history_url: Optional[str] = None
    """URL to view conversation history."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import report_list_mdrs_params, report_list_wdrs_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from .cdr_usage_reports import (
    CdrUsageReportsResource,
    AsyncCdrUsageReportsResource,
    CdrUsageReportsResourceWithRawResponse,
    AsyncCdrUsageReportsResourceWithRawResponse,
    CdrUsageReportsResourceWithStreamingResponse,
    AsyncCdrUsageReportsResourceWithStreamingResponse,
)
from .mdr_usage_reports import (
    MdrUsageReportsResource,
    AsyncMdrUsageReportsResource,
    MdrUsageReportsResourceWithRawResponse,
    AsyncMdrUsageReportsResourceWithRawResponse,
    MdrUsageReportsResourceWithStreamingResponse,
    AsyncMdrUsageReportsResourceWithStreamingResponse,
)
from ...types.report_list_mdrs_response import ReportListMdrsResponse
from ...types.report_list_wdrs_response import ReportListWdrsResponse

__all__ = ["ReportsResource", "AsyncReportsResource"]


class ReportsResource(SyncAPIResource):
    @cached_property
    def cdr_usage_reports(self) -> CdrUsageReportsResource:
        """Voice usage reports"""
        return CdrUsageReportsResource(self._client)

    @cached_property
    def mdr_usage_reports(self) -> MdrUsageReportsResource:
        """Messaging usage reports"""
        return MdrUsageReportsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ReportsResourceWithStreamingResponse(self)

    def list_mdrs(
        self,
        *,
        id: str | Omit = omit,
        cld: str | Omit = omit,
        cli: str | Omit = omit,
        direction: Literal["INBOUND", "OUTBOUND"] | Omit = omit,
        end_date: str | Omit = omit,
        message_type: Literal["SMS", "MMS"] | Omit = omit,
        profile: str | Omit = omit,
        start_date: str | Omit = omit,
        status: Literal["GW_TIMEOUT", "DELIVERED", "DLR_UNCONFIRMED", "DLR_TIMEOUT", "RECEIVED", "GW_REJECT", "FAILED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportListMdrsResponse:
        """
        Fetch all Mdr records

        Args:
          id: Message uuid

          cld: Destination number

          cli: Origination number

          direction: Direction (inbound or outbound)

          end_date: Pagination end date

          message_type: Type of message

          profile: Name of the profile

          start_date: Pagination start date

          status: Message status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/reports/mdrs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "cld": cld,
                        "cli": cli,
                        "direction": direction,
                        "end_date": end_date,
                        "message_type": message_type,
                        "profile": profile,
                        "start_date": start_date,
                        "status": status,
                    },
                    report_list_mdrs_params.ReportListMdrsParams,
                ),
            ),
            cast_to=ReportListMdrsResponse,
        )

    def list_wdrs(
        self,
        *,
        id: str | Omit = omit,
        end_date: str | Omit = omit,
        imsi: str | Omit = omit,
        mcc: str | Omit = omit,
        mnc: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        phone_number: str | Omit = omit,
        sim_card_id: str | Omit = omit,
        sim_group_id: str | Omit = omit,
        sim_group_name: str | Omit = omit,
        sort: SequenceNotStr[str] | Omit = omit,
        start_date: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ReportListWdrsResponse]:
        """
        Fetch all Wdr records

        Args:
          id: WDR uuid

          end_date: End date

          imsi: International mobile subscriber identity

          mcc: Mobile country code

          mnc: Mobile network code

          phone_number: Phone number

          sim_card_id: Sim card unique identifier

          sim_group_id: Sim group unique identifier

          sim_group_name: Sim group name

          sort: Field used to order the data. If no field is specified, default value is
              'created_at'

          start_date: Start date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/reports/wdrs",
            page=SyncDefaultFlatPagination[ReportListWdrsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "end_date": end_date,
                        "imsi": imsi,
                        "mcc": mcc,
                        "mnc": mnc,
                        "page_number": page_number,
                        "page_size": page_size,
                        "phone_number": phone_number,
                        "sim_card_id": sim_card_id,
                        "sim_group_id": sim_group_id,
                        "sim_group_name": sim_group_name,
                        "sort": sort,
                        "start_date": start_date,
                    },
                    report_list_wdrs_params.ReportListWdrsParams,
                ),
            ),
            model=ReportListWdrsResponse,
        )


class AsyncReportsResource(AsyncAPIResource):
    @cached_property
    def cdr_usage_reports(self) -> AsyncCdrUsageReportsResource:
        """Voice usage reports"""
        return AsyncCdrUsageReportsResource(self._client)

    @cached_property
    def mdr_usage_reports(self) -> AsyncMdrUsageReportsResource:
        """Messaging usage reports"""
        return AsyncMdrUsageReportsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncReportsResourceWithStreamingResponse(self)

    async def list_mdrs(
        self,
        *,
        id: str | Omit = omit,
        cld: str | Omit = omit,
        cli: str | Omit = omit,
        direction: Literal["INBOUND", "OUTBOUND"] | Omit = omit,
        end_date: str | Omit = omit,
        message_type: Literal["SMS", "MMS"] | Omit = omit,
        profile: str | Omit = omit,
        start_date: str | Omit = omit,
        status: Literal["GW_TIMEOUT", "DELIVERED", "DLR_UNCONFIRMED", "DLR_TIMEOUT", "RECEIVED", "GW_REJECT", "FAILED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReportListMdrsResponse:
        """
        Fetch all Mdr records

        Args:
          id: Message uuid

          cld: Destination number

          cli: Origination number

          direction: Direction (inbound or outbound)

          end_date: Pagination end date

          message_type: Type of message

          profile: Name of the profile

          start_date: Pagination start date

          status: Message status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/reports/mdrs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "cld": cld,
                        "cli": cli,
                        "direction": direction,
                        "end_date": end_date,
                        "message_type": message_type,
                        "profile": profile,
                        "start_date": start_date,
                        "status": status,
                    },
                    report_list_mdrs_params.ReportListMdrsParams,
                ),
            ),
            cast_to=ReportListMdrsResponse,
        )

    def list_wdrs(
        self,
        *,
        id: str | Omit = omit,
        end_date: str | Omit = omit,
        imsi: str | Omit = omit,
        mcc: str | Omit = omit,
        mnc: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        phone_number: str | Omit = omit,
        sim_card_id: str | Omit = omit,
        sim_group_id: str | Omit = omit,
        sim_group_name: str | Omit = omit,
        sort: SequenceNotStr[str] | Omit = omit,
        start_date: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ReportListWdrsResponse, AsyncDefaultFlatPagination[ReportListWdrsResponse]]:
        """
        Fetch all Wdr records

        Args:
          id: WDR uuid

          end_date: End date

          imsi: International mobile subscriber identity

          mcc: Mobile country code

          mnc: Mobile network code

          phone_number: Phone number

          sim_card_id: Sim card unique identifier

          sim_group_id: Sim group unique identifier

          sim_group_name: Sim group name

          sort: Field used to order the data. If no field is specified, default value is
              'created_at'

          start_date: Start date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/reports/wdrs",
            page=AsyncDefaultFlatPagination[ReportListWdrsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "end_date": end_date,
                        "imsi": imsi,
                        "mcc": mcc,
                        "mnc": mnc,
                        "page_number": page_number,
                        "page_size": page_size,
                        "phone_number": phone_number,
                        "sim_card_id": sim_card_id,
                        "sim_group_id": sim_group_id,
                        "sim_group_name": sim_group_name,
                        "sort": sort,
                        "start_date": start_date,
                    },
                    report_list_wdrs_params.ReportListWdrsParams,
                ),
            ),
            model=ReportListWdrsResponse,
        )


class ReportsResourceWithRawResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.list_mdrs = to_raw_response_wrapper(
            reports.list_mdrs,
        )
        self.list_wdrs = to_raw_response_wrapper(
            reports.list_wdrs,
        )

    @cached_property
    def cdr_usage_reports(self) -> CdrUsageReportsResourceWithRawResponse:
        """Voice usage reports"""
        return CdrUsageReportsResourceWithRawResponse(self._reports.cdr_usage_reports)

    @cached_property
    def mdr_usage_reports(self) -> MdrUsageReportsResourceWithRawResponse:
        """Messaging usage reports"""
        return MdrUsageReportsResourceWithRawResponse(self._reports.mdr_usage_reports)


class AsyncReportsResourceWithRawResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.list_mdrs = async_to_raw_response_wrapper(
            reports.list_mdrs,
        )
        self.list_wdrs = async_to_raw_response_wrapper(
            reports.list_wdrs,
        )

    @cached_property
    def cdr_usage_reports(self) -> AsyncCdrUsageReportsResourceWithRawResponse:
        """Voice usage reports"""
        return AsyncCdrUsageReportsResourceWithRawResponse(self._reports.cdr_usage_reports)

    @cached_property
    def mdr_usage_reports(self) -> AsyncMdrUsageReportsResourceWithRawResponse:
        """Messaging usage reports"""
        return AsyncMdrUsageReportsResourceWithRawResponse(self._reports.mdr_usage_reports)


class ReportsResourceWithStreamingResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.list_mdrs = to_streamed_response_wrapper(
            reports.list_mdrs,
        )
        self.list_wdrs = to_streamed_response_wrapper(
            reports.list_wdrs,
        )

    @cached_property
    def cdr_usage_reports(self) -> CdrUsageReportsResourceWithStreamingResponse:
        """Voice usage reports"""
        return CdrUsageReportsResourceWithStreamingResponse(self._reports.cdr_usage_reports)

    @cached_property
    def mdr_usage_reports(self) -> MdrUsageReportsResourceWithStreamingResponse:
        """Messaging usage reports"""
        return MdrUsageReportsResourceWithStreamingResponse(self._reports.mdr_usage_reports)


class AsyncReportsResourceWithStreamingResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.list_mdrs = async_to_streamed_response_wrapper(
            reports.list_mdrs,
        )
        self.list_wdrs = async_to_streamed_response_wrapper(
            reports.list_wdrs,
        )

    @cached_property
    def cdr_usage_reports(self) -> AsyncCdrUsageReportsResourceWithStreamingResponse:
        """Voice usage reports"""
        return AsyncCdrUsageReportsResourceWithStreamingResponse(self._reports.cdr_usage_reports)

    @cached_property
    def mdr_usage_reports(self) -> AsyncMdrUsageReportsResourceWithStreamingResponse:
        """Messaging usage reports"""
        return AsyncMdrUsageReportsResourceWithStreamingResponse(self._reports.mdr_usage_reports)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionEndConferenceParams"]


class ActionEndConferenceParams(TypedDict, total=False):
    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    conference.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.porting_orders import additional_document_list_params, additional_document_create_params
from ...types.porting_orders.additional_document_list_response import AdditionalDocumentListResponse
from ...types.porting_orders.additional_document_create_response import AdditionalDocumentCreateResponse

__all__ = ["AdditionalDocumentsResource", "AsyncAdditionalDocumentsResource"]


class AdditionalDocumentsResource(SyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AdditionalDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AdditionalDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdditionalDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AdditionalDocumentsResourceWithStreamingResponse(self)

    def create(
        self,
        id: str,
        *,
        additional_documents: Iterable[additional_document_create_params.AdditionalDocument] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdditionalDocumentCreateResponse:
        """
        Creates a list of additional documents for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/additional_documents",
            body=maybe_transform(
                {"additional_documents": additional_documents},
                additional_document_create_params.AdditionalDocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdditionalDocumentCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: additional_document_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: additional_document_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[AdditionalDocumentListResponse]:
        """
        Returns a list of additional documents for a porting order.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[document_type]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/additional_documents",
            page=SyncDefaultFlatPagination[AdditionalDocumentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    additional_document_list_params.AdditionalDocumentListParams,
                ),
            ),
            model=AdditionalDocumentListResponse,
        )

    def delete(
        self,
        additional_document_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an additional document for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not additional_document_id:
            raise ValueError(
                f"Expected a non-empty value for `additional_document_id` but received {additional_document_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/porting_orders/{id}/additional_documents/{additional_document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAdditionalDocumentsResource(AsyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AsyncAdditionalDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAdditionalDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdditionalDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAdditionalDocumentsResourceWithStreamingResponse(self)

    async def create(
        self,
        id: str,
        *,
        additional_documents: Iterable[additional_document_create_params.AdditionalDocument] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdditionalDocumentCreateResponse:
        """
        Creates a list of additional documents for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/additional_documents",
            body=await async_maybe_transform(
                {"additional_documents": additional_documents},
                additional_document_create_params.AdditionalDocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdditionalDocumentCreateResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: additional_document_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: additional_document_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AdditionalDocumentListResponse, AsyncDefaultFlatPagination[AdditionalDocumentListResponse]]:
        """
        Returns a list of additional documents for a porting order.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[document_type]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/porting_orders/{id}/additional_documents",
            page=AsyncDefaultFlatPagination[AdditionalDocumentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    additional_document_list_params.AdditionalDocumentListParams,
                ),
            ),
            model=AdditionalDocumentListResponse,
        )

    async def delete(
        self,
        additional_document_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an additional document for a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not additional_document_id:
            raise ValueError(
                f"Expected a non-empty value for `additional_document_id` but received {additional_document_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/porting_orders/{id}/additional_documents/{additional_document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AdditionalDocumentsResourceWithRawResponse:
    def __init__(self, additional_documents: AdditionalDocumentsResource) -> None:
        self._additional_documents = additional_documents

        self.create = to_raw_response_wrapper(
            additional_documents.create,
        )
        self.list = to_raw_response_wrapper(
            additional_documents.list,
        )
        self.delete = to_raw_response_wrapper(
            additional_documents.delete,
        )


class AsyncAdditionalDocumentsResourceWithRawResponse:
    def __init__(self, additional_documents: AsyncAdditionalDocumentsResource) -> None:
        self._additional_documents = additional_documents

        self.create = async_to_raw_response_wrapper(
            additional_documents.create,
        )
        self.list = async_to_raw_response_wrapper(
            additional_documents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            additional_documents.delete,
        )


class AdditionalDocumentsResourceWithStreamingResponse:
    def __init__(self, additional_documents: AdditionalDocumentsResource) -> None:
        self._additional_documents = additional_documents

        self.create = to_streamed_response_wrapper(
            additional_documents.create,
        )
        self.list = to_streamed_response_wrapper(
            additional_documents.list,
        )
        self.delete = to_streamed_response_wrapper(
            additional_documents.delete,
        )


class AsyncAdditionalDocumentsResourceWithStreamingResponse:
    def __init__(self, additional_documents: AsyncAdditionalDocumentsResource) -> None:
        self._additional_documents = additional_documents

        self.create = async_to_streamed_response_wrapper(
            additional_documents.create,
        )
        self.list = async_to_streamed_response_wrapper(
            additional_documents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            additional_documents.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SessionAnalysisRetrieveParams"]


class SessionAnalysisRetrieveParams(TypedDict, total=False):
    record_type: Required[str]

    date_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """ISO 8601 timestamp to narrow index selection for faster lookups."""

    expand: Literal["record", "none"]
    """Controls what data to expand on each event node."""

    include_children: bool
    """Whether to include child events in the response."""

    max_depth: int
    """Maximum traversal depth for the event tree."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr
from .google_transcription_language import GoogleTranscriptionLanguage

__all__ = ["TranscriptionEngineGoogleConfigParam", "SpeechContext"]


class SpeechContext(TypedDict, total=False):
    boost: float
    """Boost factor for the speech context."""

    phrases: SequenceNotStr[str]


class TranscriptionEngineGoogleConfigParam(TypedDict, total=False):
    enable_speaker_diarization: bool
    """Enables speaker diarization."""

    hints: SequenceNotStr[str]
    """Hints to improve transcription accuracy."""

    interim_results: bool
    """Whether to send also interim results.

    If set to false, only final results will be sent.
    """

    language: GoogleTranscriptionLanguage
    """Language to use for speech recognition"""

    max_speaker_count: int
    """Defines maximum number of speakers in the conversation."""

    min_speaker_count: int
    """Defines minimum number of speakers in the conversation."""

    model: Literal[
        "latest_long",
        "latest_short",
        "command_and_search",
        "phone_call",
        "video",
        "default",
        "medical_conversation",
        "medical_dictation",
    ]
    """The model to use for transcription."""

    profanity_filter: bool
    """Enables profanity_filter."""

    speech_context: Iterable[SpeechContext]
    """Speech context to improve transcription accuracy."""

    transcription_engine: Literal["Google"]
    """Engine identifier for Google transcription service"""

    use_enhanced: bool
    """Enables enhanced transcription, this works for models `phone_call` and `video`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MobileVoiceConnection", "Inbound", "Outbound"]


class Inbound(BaseModel):
    channel_limit: Optional[int] = None


class Outbound(BaseModel):
    channel_limit: Optional[int] = None

    outbound_voice_profile_id: Optional[str] = None


class MobileVoiceConnection(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    active: Optional[bool] = None
    """Indicates if the connection is active."""

    connection_name: Optional[str] = None
    """The name of the connection."""

    created_at: Optional[datetime] = None

    inbound: Optional[Inbound] = None

    outbound: Optional[Outbound] = None

    record_type: Optional[Literal["mobile_voice_connection"]] = None
    """Identifies the type of the resource."""

    tags: Optional[List[str]] = None
    """A list of tags associated with the connection."""

    updated_at: Optional[datetime] = None

    webhook_api_version: Optional[Literal["1", "2"]] = None
    """The API version for webhooks."""

    webhook_event_failover_url: Optional[str] = None
    """The failover URL where webhooks are sent."""

    webhook_event_url: Optional[str] = None
    """The URL where webhooks are sent."""

    webhook_timeout_secs: Optional[int] = None
    """The timeout for webhooks in seconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["UserListParams"]


class UserListParams(TypedDict, total=False):
    filter_email: Annotated[str, PropertyInfo(alias="filter[email]")]
    """Filter by email address (partial match)"""

    filter_user_status: Annotated[Literal["enabled", "disabled", "blocked"], PropertyInfo(alias="filter[user_status]")]
    """Filter by user status"""

    include_groups: bool
    """When set to true, includes the groups array for each user in the response.

    The groups array contains objects with id and name for each group the user
    belongs to.
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load"""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ExportPortingOrdersCsvReport", "Filters"]


class Filters(BaseModel):
    """The filters to apply to the export porting order CSV report."""

    created_at_gt: Optional[datetime] = FieldInfo(alias="created_at__gt", default=None)
    """The date and time the porting order was created after."""

    created_at_lt: Optional[datetime] = FieldInfo(alias="created_at__lt", default=None)
    """The date and time the porting order was created before."""

    customer_reference_in: Optional[List[str]] = FieldInfo(alias="customer_reference__in", default=None)
    """The customer reference of the porting orders to include in the report."""

    status_in: Optional[
        List[
            Literal[
                "draft",
                "in-process",
                "submitted",
                "exception",
                "foc-date-confirmed",
                "cancel-pending",
                "ported",
                "cancelled",
            ]
        ]
    ] = FieldInfo(alias="status__in", default=None)
    """The status of the porting orders to include in the report."""


class ExportPortingOrdersCsvReport(BaseModel):
    """The parameters for generating a porting orders CSV report."""

    filters: Filters
    """The filters to apply to the export porting order CSV report."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["InsightListParams"]


class InsightListParams(TypedDict, total=False):
    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AccountRetrieveRecordingsJsonParams"]


class AccountRetrieveRecordingsJsonParams(TypedDict, total=False):
    date_created: Annotated[Union[str, datetime], PropertyInfo(alias="DateCreated", format="iso8601")]
    """Filters recording by the creation date.

    Expected format is ISO8601 date or date-time, ie. {YYYY}-{MM}-{DD} or
    {YYYY}-{MM}-{DD}T{hh}:{mm}:{ss}Z. Also accepts inequality operators, e.g.
    DateCreated>=2023-05-22.
    """

    page: Annotated[int, PropertyInfo(alias="Page")]
    """
    The number of the page to be displayed, zero-indexed, should be used in
    conjuction with PageToken.
    """

    page_size: Annotated[int, PropertyInfo(alias="PageSize")]
    """The number of records to be displayed on a page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["QueueUpdateParams"]


class QueueUpdateParams(TypedDict, total=False):
    account_sid: Required[str]

    max_size: Annotated[int, PropertyInfo(alias="MaxSize")]
    """The maximum size of the queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ExternalConnectionListParams", "Filter", "FilterConnectionName", "FilterPhoneNumber"]


class ExternalConnectionListParams(TypedDict, total=False):
    filter: Filter
    """Filter parameter for external connections (deepObject style).

    Supports filtering by connection_name, external_sip_connection, id, created_at,
    and phone_number.
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterConnectionName(TypedDict, total=False):
    contains: str
    """
    If present, connections with <code>connection_name</code> containing the given
    value will be returned. Matching is not case-sensitive. Requires at least three
    characters.
    """


class FilterPhoneNumber(TypedDict, total=False):
    """Phone number filter for connections.

    Note: Despite the 'contains' name, this requires a full E164 match per the original specification.
    """

    contains: str
    """If present, connections associated with the given phone_number will be returned.

    A full match is necessary with a e164 format.
    """


class Filter(TypedDict, total=False):
    """Filter parameter for external connections (deepObject style).

    Supports filtering by connection_name, external_sip_connection, id, created_at, and phone_number.
    """

    id: str
    """
    If present, connections with <code>id</code> matching the given value will be
    returned.
    """

    connection_name: FilterConnectionName

    created_at: str
    """
    If present, connections with <code>created_at</code> date matching the given
    YYYY-MM-DD date will be returned.
    """

    external_sip_connection: Literal["zoom", "operator_connect"]
    """
    If present, connections with <code>external_sip_connection</code> matching the
    given value will be returned.
    """

    phone_number: FilterPhoneNumber
    """Phone number filter for connections.

    Note: Despite the 'contains' name, this requires a full E164 match per the
    original specification.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import connection_list_params, connection_list_active_calls_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.connection_list_response import ConnectionListResponse
from ..types.connection_retrieve_response import ConnectionRetrieveResponse
from ..types.connection_list_active_calls_response import ConnectionListActiveCallsResponse

__all__ = ["ConnectionsResource", "AsyncConnectionsResource"]


class ConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ConnectionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveResponse:
        """Retrieves the high-level details of an existing connection.

        To retrieve specific
        authentication information, use the endpoint for the specific connection type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ConnectionListResponse]:
        """
        Returns a list of your connections irrespective of type.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/connections",
            page=SyncDefaultFlatPagination[ConnectionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    connection_list_params.ConnectionListParams,
                ),
            ),
            model=ConnectionListResponse,
        )

    def list_active_calls(
        self,
        connection_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ConnectionListActiveCallsResponse]:
        """Lists all active calls for given connection.

        Acceptable connections are either
        SIP connections with webhook_url or xml_request_url, call control or texml.
        Returned results are cursor paginated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._get_api_list(
            f"/connections/{connection_id}/active_calls",
            page=SyncDefaultFlatPagination[ConnectionListActiveCallsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    connection_list_active_calls_params.ConnectionListActiveCallsParams,
                ),
            ),
            model=ConnectionListActiveCallsResponse,
        )


class AsyncConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncConnectionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveResponse:
        """Retrieves the high-level details of an existing connection.

        To retrieve specific
        authentication information, use the endpoint for the specific connection type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "connection_name", "active"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ConnectionListResponse, AsyncDefaultFlatPagination[ConnectionListResponse]]:
        """
        Returns a list of your connections irrespective of type.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[connection_name], filter[fqdn], filter[outbound_voice_profile_id],
              filter[outbound.outbound_voice_profile_id]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in ascending order.
                </li>

                <li>
                  <code>-connection_name</code>: sorts the result by the
                  <code>connection_name</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/connections",
            page=AsyncDefaultFlatPagination[ConnectionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    connection_list_params.ConnectionListParams,
                ),
            ),
            model=ConnectionListResponse,
        )

    def list_active_calls(
        self,
        connection_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        ConnectionListActiveCallsResponse, AsyncDefaultFlatPagination[ConnectionListActiveCallsResponse]
    ]:
        """Lists all active calls for given connection.

        Acceptable connections are either
        SIP connections with webhook_url or xml_request_url, call control or texml.
        Returned results are cursor paginated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._get_api_list(
            f"/connections/{connection_id}/active_calls",
            page=AsyncDefaultFlatPagination[ConnectionListActiveCallsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    connection_list_active_calls_params.ConnectionListActiveCallsParams,
                ),
            ),
            model=ConnectionListActiveCallsResponse,
        )


class ConnectionsResourceWithRawResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = to_raw_response_wrapper(
            connections.retrieve,
        )
        self.list = to_raw_response_wrapper(
            connections.list,
        )
        self.list_active_calls = to_raw_response_wrapper(
            connections.list_active_calls,
        )


class AsyncConnectionsResourceWithRawResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = async_to_raw_response_wrapper(
            connections.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            connections.list,
        )
        self.list_active_calls = async_to_raw_response_wrapper(
            connections.list_active_calls,
        )


class ConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            connections.list,
        )
        self.list_active_calls = to_streamed_response_wrapper(
            connections.list_active_calls,
        )


class AsyncConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = async_to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            connections.list,
        )
        self.list_active_calls = async_to_streamed_response_wrapper(
            connections.list_active_calls,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.assistants import tool_test_params
from ....types.ai.assistants.tool_test_response import ToolTestResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def test(
        self,
        tool_id: str,
        *,
        assistant_id: str,
        arguments: Dict[str, object] | Omit = omit,
        dynamic_variables: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolTestResponse:
        """
        Test a webhook tool for an assistant

        Args:
          arguments: Key-value arguments to use for the webhook test

          dynamic_variables: Key-value dynamic variables to use for the webhook test

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._post(
            f"/ai/assistants/{assistant_id}/tools/{tool_id}/test",
            body=maybe_transform(
                {
                    "arguments": arguments,
                    "dynamic_variables": dynamic_variables,
                },
                tool_test_params.ToolTestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolTestResponse,
        )


class AsyncToolsResource(AsyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    async def test(
        self,
        tool_id: str,
        *,
        assistant_id: str,
        arguments: Dict[str, object] | Omit = omit,
        dynamic_variables: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolTestResponse:
        """
        Test a webhook tool for an assistant

        Args:
          arguments: Key-value arguments to use for the webhook test

          dynamic_variables: Key-value dynamic variables to use for the webhook test

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._post(
            f"/ai/assistants/{assistant_id}/tools/{tool_id}/test",
            body=await async_maybe_transform(
                {
                    "arguments": arguments,
                    "dynamic_variables": dynamic_variables,
                },
                tool_test_params.ToolTestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolTestResponse,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.test = to_raw_response_wrapper(
            tools.test,
        )


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.test = async_to_raw_response_wrapper(
            tools.test,
        )


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.test = to_streamed_response_wrapper(
            tools.test,
        )


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.test = async_to_streamed_response_wrapper(
            tools.test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.actions import RegisterCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRegister:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        register = client.actions.register.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        )
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        register = client.actions.register.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
            sim_card_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            status="standby",
            tags=["personal", "customers", "active-customers"],
        )
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.actions.register.with_raw_response.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        register = response.parse()
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.actions.register.with_streaming_response.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            register = response.parse()
            assert_matches_type(RegisterCreateResponse, register, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRegister:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        register = await async_client.actions.register.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        )
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        register = await async_client.actions.register.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
            sim_card_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            status="standby",
            tags=["personal", "customers", "active-customers"],
        )
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.actions.register.with_raw_response.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        register = await response.parse()
        assert_matches_type(RegisterCreateResponse, register, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.actions.register.with_streaming_response.create(
            registration_codes=["0000000001", "0000000002", "0000000003"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            register = await response.parse()
            assert_matches_type(RegisterCreateResponse, register, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AssistantChatParams"]


class AssistantChatParams(TypedDict, total=False):
    content: Required[str]
    """The message content sent by the client to the assistant"""

    conversation_id: Required[str]
    """A unique identifier for the conversation thread, used to maintain context"""

    name: str
    """The optional display name of the user sending the message"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IPListParams", "Filter"]


class IPListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[connection_id], filter[ip_address], filter[port]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[connection_id], filter[ip_address], filter[port]
    """

    connection_id: str
    """ID of the IP Connection to which this IP should be attached."""

    ip_address: str
    """IP adddress represented by this resource."""

    port: int
    """Port to use when connecting to this IP."""
